@@ -12,6 +12,8 @@ static const R_CallMethodDef callMethods[] = {
   {"secretbase_base64dec", (DL_FUNC) &secretbase_base64dec, 2},
   {"secretbase_base58enc", (DL_FUNC) &secretbase_base58enc, 2},
   {"secretbase_base58dec", (DL_FUNC) &secretbase_base58dec, 2},
+  {"secretbase_jsonenc", (DL_FUNC) &secretbase_jsonenc, 1},
+  {"secretbase_jsondec", (DL_FUNC) &secretbase_jsondec, 1},
   {"secretbase_sha3", (DL_FUNC) &secretbase_sha3, 3},
   {"secretbase_sha3_file", (DL_FUNC) &secretbase_sha3_file, 3},
   {"secretbase_shake256", (DL_FUNC) &secretbase_shake256, 3},
